@@ -25,16 +25,10 @@
 #include "avsutils.h"
 
 static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()),  Args[7].AsBool(false));
-
-	char ErrorMsg[1024];
-	FFMS_ErrorInfo E;
-	E.Buffer = ErrorMsg;
-	E.BufferSize = sizeof(ErrorMsg);
-
-
 	if (!Args[0].Defined())
-    	Env->ThrowError("FFIndex: No source specified");
+		Env->ThrowError("FFIndex: No source specified");
+
+	FFMS_Init(0, Args[7].AsBool(false));
 
 	const char *Source = Args[0].AsString();
 	const char *CacheFile = Args[1].AsString("");
@@ -67,6 +61,7 @@ static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvi
 	else
 		Env->ThrowError("FFIndex: Invalid demuxer requested");
 
+	ErrorInfo E;
 	FFMS_Index *Index = FFMS_ReadIndex(CacheFile, &E);
 	if (OverWrite || !Index || (Index && FFMS_IndexBelongsToFile(Index, Source, 0) != FFMS_ERROR_SUCCESS)) {
 		FFMS_Indexer *Indexer = FFMS_CreateIndexerWithDemuxer(Source, Demuxer, &E);
@@ -90,15 +85,10 @@ static AVSValue __cdecl CreateFFIndex(AVSValue Args, void* UserData, IScriptEnvi
 }
 
 static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()), Args[15].AsBool(false));
-
-	char ErrorMsg[1024];
-	FFMS_ErrorInfo E;
-	E.Buffer = ErrorMsg;
-	E.BufferSize = sizeof(ErrorMsg);
+	FFMS_Init(0, Args[15].AsBool(false));
 
 	if (!Args[0].Defined())
-    	Env->ThrowError("FFVideoSource: No source specified");
+		Env->ThrowError("FFVideoSource: No source specified");
 
 	const char *Source = Args[0].AsString();
 	int Track = Args[1].AsInt(-1);
@@ -134,6 +124,7 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 	if (!_stricmp(Source, Timecodes))
 		Env->ThrowError("FFVideoSource: Timecodes will overwrite the source");
 
+	ErrorInfo E;
 	FFMS_Index *Index = NULL;
 	std::string DefaultCache;
 	if (Cache) {
@@ -193,15 +184,10 @@ static AVSValue __cdecl CreateFFVideoSource(AVSValue Args, void* UserData, IScri
 }
 
 static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScriptEnvironment* Env) {
-	FFMS_Init((int)AvisynthToFFCPUFlags(Env->GetCPUFlags()), Args[5].AsBool(false));
-
-	char ErrorMsg[1024];
-	FFMS_ErrorInfo E;
-	E.Buffer = ErrorMsg;
-	E.BufferSize = sizeof(ErrorMsg);
+	FFMS_Init(0, Args[5].AsBool(false));
 
 	if (!Args[0].Defined())
-    	Env->ThrowError("FFAudioSource: No source specified");
+		Env->ThrowError("FFAudioSource: No source specified");
 
 	const char *Source = Args[0].AsString();
 	int Track = Args[1].AsInt(-1);
@@ -213,6 +199,7 @@ static AVSValue __cdecl CreateFFAudioSource(AVSValue Args, void* UserData, IScri
 	if (Track <= -2)
 		Env->ThrowError("FFAudioSource: No audio track selected");
 
+	ErrorInfo E;
 	FFMS_Index *Index = NULL;
 	std::string DefaultCache;
 	if (Cache) {
@@ -308,7 +295,7 @@ static AVSValue __cdecl FFGetVersion(AVSValue Args, void* UserData, IScriptEnvir
 }
 
 extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* Env) {
-    Env->AddFunction("FFIndex", "[source]s[cachefile]s[indexmask]i[dumpmask]i[audiofile]s[errorhandling]i[overwrite]b[utf8]b[demuxer]s", CreateFFIndex, 0);
+	Env->AddFunction("FFIndex", "[source]s[cachefile]s[indexmask]i[dumpmask]i[audiofile]s[errorhandling]i[overwrite]b[utf8]b[demuxer]s", CreateFFIndex, 0);
 	Env->AddFunction("FFVideoSource", "[source]s[track]i[cache]b[cachefile]s[fpsnum]i[fpsden]i[threads]i[timecodes]s[seekmode]i[rffmode]i[width]i[height]i[resizer]s[colorspace]s[utf8]b[varprefix]s", CreateFFVideoSource, 0);
 	Env->AddFunction("FFAudioSource", "[source]s[track]i[cache]b[cachefile]s[adjustdelay]i[utf8]b[varprefix]s", CreateFFAudioSource, 0);
 	Env->AddFunction("SWScale", "c[width]i[height]i[resizer]s[colorspace]s", CreateSWScale, 0);
@@ -316,5 +303,5 @@ extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScri
 	Env->AddFunction("FFSetLogLevel", "i", FFSetLogLevel, 0);
 	Env->AddFunction("FFGetVersion", "", FFGetVersion, 0);
 
-    return "FFmpegSource - The Second Coming V2.0 Final";
+	return "FFmpegSource - The Second Coming V2.0 Final";
 }
