@@ -18,35 +18,12 @@
 //  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 //  THE SOFTWARE.
 
-#include <windows.h>
-#include "avisynth.h"
-#include "ffms.h"
+#include "avsutils.h"
 
 extern "C" {
 #include <libswscale/swscale.h>
 }
 
-
-
-int64_t AvisynthToFFCPUFlags(long AvisynthFlags, bool ForFFMS) {
-	int64_t Flags = 0;
-#define CPU_FLAG(FLAG) ForFFMS ? FFMS_CPU_CAPS_##FLAG : SWS_CPU_CAPS_##FLAG
-
-	if (AvisynthFlags & CPUF_MMX)
-		Flags |= CPU_FLAG(MMX);
-	if (AvisynthFlags & CPUF_INTEGER_SSE)
-		Flags |= CPU_FLAG(MMX2);
-	if (AvisynthFlags & CPUF_3DNOW_EXT)
-		Flags |= CPU_FLAG(3DNOW);
-#ifdef SWS_CPU_CAPS_SSE2
-	if (AvisynthFlags & CPUF_SSE2)
-		Flags |= CPU_FLAG(SSE2);
-#endif
-
-#undef CPU_FLAG
-	return Flags;
-}
-
 PixelFormat CSNameToPIXFMT(const char *CSName, PixelFormat Default) {
 	if (!_stricmp(CSName, ""))
 		return Default;
