@@ -29,66 +29,73 @@ typedef struct CodecTags {
 } CodecTags;
 
 static const CodecTags mkv_codec_tags[] = {
-	{"A_AAC"            , FFMS_ID(AAC)},
-	{"A_AC3"            , FFMS_ID(AC3)},
-	{"A_DTS"            , FFMS_ID(DTS)},
-	{"A_EAC3"           , FFMS_ID(EAC3)},
-	{"A_FLAC"           , FFMS_ID(FLAC)},
-	{"A_MLP"            , FFMS_ID(MLP)},
-	{"A_MPEG/L2"        , FFMS_ID(MP2)},
-	{"A_MPEG/L1"        , FFMS_ID(MP2)},
-	{"A_MPEG/L3"        , FFMS_ID(MP3)},
-	{"A_PCM/FLOAT/IEEE" , FFMS_ID(PCM_F32LE)},
-	{"A_PCM/FLOAT/IEEE" , FFMS_ID(PCM_F64LE)},
-	{"A_PCM/INT/BIG"    , FFMS_ID(PCM_S16BE)},
-	{"A_PCM/INT/BIG"    , FFMS_ID(PCM_S24BE)},
-	{"A_PCM/INT/BIG"    , FFMS_ID(PCM_S32BE)},
-	{"A_PCM/INT/LIT"    , FFMS_ID(PCM_S16LE)},
-	{"A_PCM/INT/LIT"    , FFMS_ID(PCM_S24LE)},
-	{"A_PCM/INT/LIT"    , FFMS_ID(PCM_S32LE)},
-	{"A_PCM/INT/LIT"    , FFMS_ID(PCM_U8)},
-	{"A_QUICKTIME/QDM2" , FFMS_ID(QDM2)},
-	{"A_REAL/14_4"      , FFMS_ID(RA_144)},
-	{"A_REAL/28_8"      , FFMS_ID(RA_288)},
-	{"A_REAL/ATRC"      , FFMS_ID(ATRAC3)},
-	{"A_REAL/COOK"      , FFMS_ID(COOK)},
-	{"A_REAL/SIPR"      , FFMS_ID(SIPR)},
-	{"A_TRUEHD"         , FFMS_ID(TRUEHD)},
-	{"A_TTA1"           , FFMS_ID(TTA)},
-	{"A_VORBIS"         , FFMS_ID(VORBIS)},
-	{"A_WAVPACK4"       , FFMS_ID(WAVPACK)},
-
-	{"S_TEXT/UTF8"      , FFMS_ID(TEXT)},
-	{"S_TEXT/UTF8"      , FFMS_ID(SRT)},
-	{"S_TEXT/ASCII"     , FFMS_ID(TEXT)},
-	{"S_TEXT/ASS"       , FFMS_ID(SSA)},
-	{"S_TEXT/SSA"       , FFMS_ID(SSA)},
-	{"S_ASS"            , FFMS_ID(SSA)},
-	{"S_SSA"            , FFMS_ID(SSA)},
-	{"S_VOBSUB"         , FFMS_ID(DVD_SUBTITLE)},
-	{"S_HDMV/PGS"       , FFMS_ID(HDMV_PGS_SUBTITLE)},
-
-	{"V_DIRAC"          , FFMS_ID(DIRAC)},
-	{"V_MJPEG"          , FFMS_ID(MJPEG)},
-	{"V_MPEG1"          , FFMS_ID(MPEG1VIDEO)},
-	{"V_MPEG2"          , FFMS_ID(MPEG2VIDEO)},
-	{"V_MPEG4/ISO/ASP"  , FFMS_ID(MPEG4)},
-	{"V_MPEG4/ISO/AP"   , FFMS_ID(MPEG4)},
-	{"V_MPEG4/ISO/SP"   , FFMS_ID(MPEG4)},
-	{"V_MPEG4/ISO/AVC"  , FFMS_ID(H264)},
-	{"V_MPEG4/MS/V3"    , FFMS_ID(MSMPEG4V3)},
-	{"V_REAL/RV10"      , FFMS_ID(RV10)},
-	{"V_REAL/RV20"      , FFMS_ID(RV20)},
-	{"V_REAL/RV30"      , FFMS_ID(RV30)},
-	{"V_REAL/RV40"      , FFMS_ID(RV40)},
-#if LIBAVCODEC_VERSION_MAJOR < 55
-	{"V_SNOW"           , FFMS_ID(SNOW)},
+	{"A_AAC"               , FFMS_ID(AAC)},
+	{"A_AC3"               , FFMS_ID(AC3)},
+	{"A_DTS"               , FFMS_ID(DTS)},
+	{"A_EAC3"              , FFMS_ID(EAC3)},
+	{"A_FLAC"              , FFMS_ID(FLAC)},
+	{"A_MLP"               , FFMS_ID(MLP)},
+	{"A_MPEG/L2"           , FFMS_ID(MP2)},
+	{"A_MPEG/L1"           , FFMS_ID(MP2)},
+	{"A_MPEG/L3"           , FFMS_ID(MP3)},
+#if VERSION_CHECK(LIBAVCODEC_VERSION_INT, >=, 54, 29, 0, 54, 60, 100)
+	{"A_OPUS"              , FFMS_ID(OPUS)},
+	{"A_OPUS/EXPERIMENTAL" , FFMS_ID(OPUS)}, /* Pre-final ID, for compatibility */
 #endif
-	{"V_THEORA"         , FFMS_ID(THEORA)},
-	{"V_UNCOMPRESSED"   , FFMS_ID(RAWVIDEO)},
-	{"V_VP8"            , FFMS_ID(VP8)},
+	{"A_PCM/FLOAT/IEEE"    , FFMS_ID(PCM_F32LE)},
+	{"A_PCM/FLOAT/IEEE"    , FFMS_ID(PCM_F64LE)},
+	{"A_PCM/INT/BIG"       , FFMS_ID(PCM_S16BE)},
+	{"A_PCM/INT/BIG"       , FFMS_ID(PCM_S24BE)},
+	{"A_PCM/INT/BIG"       , FFMS_ID(PCM_S32BE)},
+	{"A_PCM/INT/LIT"       , FFMS_ID(PCM_S16LE)},
+	{"A_PCM/INT/LIT"       , FFMS_ID(PCM_S24LE)},
+	{"A_PCM/INT/LIT"       , FFMS_ID(PCM_S32LE)},
+	{"A_PCM/INT/LIT"       , FFMS_ID(PCM_U8)},
+	{"A_QUICKTIME/QDM2"    , FFMS_ID(QDM2)},
+	{"A_REAL/14_4"         , FFMS_ID(RA_144)},
+	{"A_REAL/28_8"         , FFMS_ID(RA_288)},
+	{"A_REAL/ATRC"         , FFMS_ID(ATRAC3)},
+	{"A_REAL/COOK"         , FFMS_ID(COOK)},
+	{"A_REAL/SIPR"         , FFMS_ID(SIPR)},
+	{"A_TRUEHD"            , FFMS_ID(TRUEHD)},
+	{"A_TTA1"              , FFMS_ID(TTA)},
+	{"A_VORBIS"            , FFMS_ID(VORBIS)},
+	{"A_WAVPACK4"          , FFMS_ID(WAVPACK)},
 
-	{""                 , FFMS_ID(NONE)}
+	{"S_TEXT/UTF8"         , FFMS_ID(TEXT)},
+	{"S_TEXT/UTF8"         , FFMS_ID(SRT)},
+	{"S_TEXT/ASCII"        , FFMS_ID(TEXT)},
+	{"S_TEXT/ASS"          , FFMS_ID(SSA)},
+	{"S_TEXT/SSA"          , FFMS_ID(SSA)},
+	{"S_ASS"               , FFMS_ID(SSA)},
+	{"S_SSA"               , FFMS_ID(SSA)},
+	{"S_VOBSUB"            , FFMS_ID(DVD_SUBTITLE)},
+	{"S_HDMV/PGS"          , FFMS_ID(HDMV_PGS_SUBTITLE)},
+
+	{"V_DIRAC"             , FFMS_ID(DIRAC)},
+	{"V_MJPEG"             , FFMS_ID(MJPEG)},
+	{"V_MPEG1"             , FFMS_ID(MPEG1VIDEO)},
+	{"V_MPEG2"             , FFMS_ID(MPEG2VIDEO)},
+	{"V_MPEG4/ISO/ASP"     , FFMS_ID(MPEG4)},
+	{"V_MPEG4/ISO/AP"      , FFMS_ID(MPEG4)},
+	{"V_MPEG4/ISO/SP"      , FFMS_ID(MPEG4)},
+	{"V_MPEG4/ISO/AVC"     , FFMS_ID(H264)},
+	{"V_MPEG4/MS/V3"       , FFMS_ID(MSMPEG4V3)},
+#if VERSION_CHECK(LIBAVCODEC_VERSION_INT, >=, 55, 24, 0, 55, 38, 101)
+	{"V_MPEGH/ISO/HEVC"    , FFMS_ID(HEVC)},
+#endif
+	{"V_REAL/RV10"         , FFMS_ID(RV10)},
+	{"V_REAL/RV20"         , FFMS_ID(RV20)},
+	{"V_REAL/RV30"         , FFMS_ID(RV30)},
+	{"V_REAL/RV40"         , FFMS_ID(RV40)},
+#if LIBAVCODEC_VERSION_MAJOR < 55
+	{"V_SNOW"              , FFMS_ID(SNOW)},
+#endif
+	{"V_THEORA"            , FFMS_ID(THEORA)},
+	{"V_UNCOMPRESSED"      , FFMS_ID(RAWVIDEO)},
+	{"V_VP8"               , FFMS_ID(VP8)},
+
+	{""                    , FFMS_ID(NONE)}
 };
 
 typedef struct AVCodecTag {
@@ -398,35 +405,33 @@ const char *GetLAVCSampleFormatName(AVSampleFormat s) {
 
 FFMS_CodecID MatroskaToFFCodecID(char *Codec, void *CodecPrivate, unsigned int FourCC, unsigned int BitsPerSample) {
 	/* Look up native codecs */
-	for(int i = 0; mkv_codec_tags[i].id != FFMS_ID(NONE); i++){
-		if(!strncmp(mkv_codec_tags[i].str, Codec,
-			strlen(mkv_codec_tags[i].str))) {
+	for (int i = 0; mkv_codec_tags[i].id != FFMS_ID(NONE); i++){
+		if (!strncmp(mkv_codec_tags[i].str, Codec, strlen(mkv_codec_tags[i].str))) {
+			// Uncompressed and exotic format fixup
+			// This list is incomplete
+			FFMS_CodecID CID = mkv_codec_tags[i].id;
+			switch (CID) {
+				case FFMS_ID(PCM_S16LE):
+					switch (BitsPerSample) {
+						case 8: CID = FFMS_ID(PCM_S8); break;
+						case 16: CID = FFMS_ID(PCM_S16LE); break;
+						case 24: CID = FFMS_ID(PCM_S24LE); break;
+						case 32: CID = FFMS_ID(PCM_S32LE); break;
+					}
+					break;
+				case FFMS_ID(PCM_S16BE):
+					switch (BitsPerSample) {
+						case 8: CID = FFMS_ID(PCM_S8); break;
+						case 16: CID = FFMS_ID(PCM_S16BE); break;
+						case 24: CID = FFMS_ID(PCM_S24BE); break;
+						case 32: CID = FFMS_ID(PCM_S32BE); break;
+					}
+					break;
+				default:
+					break;
+			}
 
-				// Uncompressed and exotic format fixup
-				// This list is incomplete
-				FFMS_CodecID CID = mkv_codec_tags[i].id;
-				switch (CID) {
-					case FFMS_ID(PCM_S16LE):
-						switch (BitsPerSample) {
-							case 8: CID = FFMS_ID(PCM_S8); break;
-							case 16: CID = FFMS_ID(PCM_S16LE); break;
-							case 24: CID = FFMS_ID(PCM_S24LE); break;
-							case 32: CID = FFMS_ID(PCM_S32LE); break;
-						}
-						break;
-					case FFMS_ID(PCM_S16BE):
-						switch (BitsPerSample) {
-							case 8: CID = FFMS_ID(PCM_S8); break;
-							case 16: CID = FFMS_ID(PCM_S16BE); break;
-							case 24: CID = FFMS_ID(PCM_S24BE); break;
-							case 32: CID = FFMS_ID(PCM_S32BE); break;
-						}
-						break;
-					default:
-						break;
-				}
-
-				return CID;
+			return CID;
 		}
 	}
 
