@@ -20,9 +20,23 @@
 
 #ifdef HAALISOURCE
 
+#include "haalicommon.h"
 #include "videosource.h"
 
+namespace {
+class FFHaaliVideo : public FFMS_VideoSource {
+	FFCodecContext HCodecContext;
+	FFSourceResources<FFMS_VideoSource> Res;
+	CComPtr<IMMContainer> pMMC;
+	AVBitStreamFilterContext *BitStreamFilter;
 
+	void DecodeNextFrame(int64_t *AFirstStartTime);
+	void Free(bool CloseCodec);
+
+public:
+	FFHaaliVideo(const char *SourceFile, int Track, FFMS_Index &Index, int Threads, FFMS_Sources SourceMode);
+	FFMS_Frame *GetFrame(int n);
+};
 
 void FFHaaliVideo::Free(bool CloseCodec) {
 	if (CloseCodec)
@@ -31,13 +45,12 @@ void FFHaaliVideo::Free(bool CloseCodec) {
 		av_bitstream_filter_close(BitStreamFilter);
 }
 
-FFHaaliVideo::FFHaaliVideo(const char *SourceFile, int Track,
-	FFMS_Index &Index, int Threads, FFMS_Sources SourceMode)
-: Res(FFSourceResources<FFMS_VideoSource>(this)), FFMS_VideoSource(SourceFile, Index, Track, Threads) {
-	BitStreamFilter = NULL;
-
-	pMMC = HaaliOpenFile(SourceFile, SourceMode);
-
+FFHaaliVideo::FFHaaliVideo(const char *SourceFile, int Track, FFMS_Index &Index, int Threads, FFMS_Sources SourceMode)
+: FFMS_VideoSource(SourceFile, Index, Track, Threads)
+, Res(this)
+, pMMC(HaaliOpenFile(SourceFile, SourceMode))
+, BitStreamFilter(NULL)
+{
 	CComPtr<IEnumUnknown> pEU;
 	if (!SUCCEEDED(pMMC->EnumTracks(&pEU)))
 		throw FFMS_Exception(FFMS_ERROR_DECODING, FFMS_ERROR_CODEC,
@@ -54,6 +67,7 @@ FFHaaliVideo::FFHaaliVideo(const char *SourceFile, int Track,
 
 	HCodecContext = InitializeCodecContextFromHaaliInfo(pBag);
 	CodecContext = HCodecContext;
+	CodecContext->has_b_frames = Frames.MaxBFrames;
 
 	const AVCodec *Codec = NULL;
 	std::swap(Codec, CodecContext->codec);
@@ -78,7 +92,7 @@ FFHaaliVideo::FFHaaliVideo(const char *SourceFile, int Track,
 	// Calculate the average framerate
 	if (Frames.size() >= 2) {
 		double PTSDiff = (double)(Frames.back().PTS - Frames.front().PTS);
-		VP.FPSDenominator = (unsigned int)(PTSDiff  / (double)1000 / (double)(Frames.size() - 1) + 0.5);
+		VP.FPSDenominator = (unsigned int)(PTSDiff / 1000.0 / (double)(Frames.size() - 1) + 0.5);
 		VP.FPSNumerator = 1000000;
 	}
 
@@ -181,7 +195,7 @@ ReSeek:
 
 	do {
 		int64_t StartTime = -1;
-		if (CurrentFrame + FFMS_CALCULATE_DELAY >= n || HasSeeked)
+		if (CurrentFrame + FFMS_CALCULATE_DELAY * CodecContext->ticks_per_frame >= n || HasSeeked)
 			CodecContext->skip_frame = AVDISCARD_DEFAULT;
 		else
 			CodecContext->skip_frame = AVDISCARD_NONREF;
@@ -207,5 +221,10 @@ ReSeek:
 	LastFrameNum = n;
 	return OutputFrame(DecodeFrame);
 }
+}
+
+FFMS_VideoSource *CreateHaaliVideoSource(const char *SourceFile, int Track, FFMS_Index &Index, int Threads, FFMS_Sources SourceMode) {
+    return new FFHaaliVideo(SourceFile, Track, Index, Threads, SourceMode);
+}
 
 #endif // HAALISOURCE
