@@ -159,6 +159,8 @@ struct MatroskaFile {
   int	      bufpos; // current read position in buffer
   int	      buflen; // valid bytes in buffer
 
+  void	      *cpbuf;
+
   // error reporting
   char	      errmsg[128];
   jmp_buf     jb;
@@ -415,6 +417,9 @@ static void   myvsnprintf(char *dest,unsigned dsize,const char *fmt,va_list ap)
 static void   errorjmp(MatroskaFile *mf,const char *fmt, ...) {
   va_list   ap;
 
+  mf->cache->memfree(mf->cache, mf->cpbuf);
+  mf->cpbuf = NULL;
+
   va_start(ap, fmt);
   myvsnprintf(mf->errmsg,sizeof(mf->errmsg),fmt,ap);
   va_end(ap);
@@ -1373,10 +1378,14 @@ static void parseTrackEntry(MatroskaFile *mf,ulonglong toplen) {
     case 0x63a2: // CodecPrivate
       if (cp)
 	errorjmp(mf,"Duplicate CodecPrivate");
-      if (len>262144) // 256KB
-	errorjmp(mf,"CodecPrivate is too large: %d",(int)len);
       cplen = (unsigned)len;
-      cp = alloca(cplen);
+      if (len > 1024) {
+	cp = mf->cpbuf = mf->cache->memalloc(mf->cache, cplen);
+	if (!cp)
+	  errorjmp(mf,"Out of memory");
+      }
+      else
+	cp = alloca(cplen);
       readbytes(mf,cp,(int)cplen);
       break;
     case 0x258688: // CodecName
@@ -1495,7 +1504,9 @@ static void parseTrackEntry(MatroskaFile *mf,ulonglong toplen) {
       errorjmp(mf, "invalid compressed data in CodecPrivate");
 
     ncplen = zs.total_out;
-    ncp = alloca(ncplen);
+    ncp = ncplen > 1024 ? mf->cache->memalloc(mf->cache, ncplen) : alloca(ncplen);
+    if (ncp == NULL)
+      errorjmp(mf,"Out of memory");
 
     inflateReset(&zs);
 
@@ -1504,13 +1515,24 @@ static void parseTrackEntry(MatroskaFile *mf,ulonglong toplen) {
     zs.next_out = ncp;
     zs.avail_out = ncplen;
 
-    if (inflate(&zs, Z_FINISH) != Z_STREAM_END)
+    if (inflate(&zs, Z_FINISH) != Z_STREAM_END) {
+      if (ncplen > 1024)
+	mf->cache->memfree(mf->cache, ncp);
       errorjmp(mf, "inflate failed");
+    }
 
     inflateEnd(&zs);
 
     cp = (char *)ncp;
     cplen = ncplen;
+
+    if (mf->cpbuf) {
+	mf->cache->memfree(mf->cache, mf->cpbuf);
+	mf->cpbuf = NULL;
+    }
+
+    if (ncplen > 1024)
+	mf->cpbuf = cp;
   }
 #endif
 
@@ -1557,9 +1579,12 @@ static void parseTrackEntry(MatroskaFile *mf,ulonglong toplen) {
 
 static void parseTracks(MatroskaFile *mf,ulonglong toplen) {
   mf->seen.Tracks = 1;
+  mf->cpbuf = NULL;
   FOREACH(mf,toplen)
     case 0xae: // TrackEntry
       parseTrackEntry(mf,len);
+      mf->cache->memfree(mf->cache, mf->cpbuf);
+      mf->cpbuf = NULL;
       break;
   ENDFOR(mf);
 }
